#include <imgui.h>
#include <imgui_stdlib.h>

#include "../config.h"
#include "settings.h"

static void write_config_and_update_structures(const Config& config) {
    try {
        write_config(config);
    } catch (const std::exception& e) {
        log(std::string("Failed to write config: ") + e.what());
        return;
    }
}

void settings_window(Config& active_config, Config& inactive_config, bool* p_open) {
    ImGuiWindowFlags flags = active_config != inactive_config
        ? ImGuiWindowFlags_UnsavedDocument
        : 0;
    if (!ImGui::Begin("Settings", p_open, flags)) {
        ImGui::End();
        return;
    }
    // TODO actually have process control
    ImGui::Text("Logcat command only takes effect when logcat is not running");
    ImGui::InputTextWithHint("Logcat command", "adb logcat -Dv 'threadtime UTC epoch usec uid'", &inactive_config.logcat_command);

    ImGui::Text("Font sizes only take effect when LogMeow is restarted");
#ifdef USE_FONTCONFIG
    ImGui::InputFloat("Normal font size", &inactive_config.normal_font_size, 0.5f, 1.0f, "%.3f");
#endif
    ImGui::InputFloat("Monospace font size", &inactive_config.monospace_font_size, 0.5f, 1.0f, "%.3f");

    ImGui::Separator();
    ImVec2 button_size(4 * ImGui::GetFontSize(), 0);
    if (ImGui::Button("OK", button_size)) {
        active_config = inactive_config;
        write_config_and_update_structures(active_config);
        *p_open = false;
    }
    ImGui::SameLine();
    if (ImGui::Button("Cancel", button_size)) {
        *p_open = false;
    }
    ImGui::SameLine();
    if (ImGui::Button("Apply", button_size)) {
        active_config = inactive_config;
        write_config_and_update_structures(active_config);
    }
    ImGui::End();
}
